/*
 * Copyright (C) 2025 Vasiliy Doylov <nekocwd@mainlining.org>
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 * Author: Vasiliy Doylov <nekocwd@mainlining.org>
 */

namespace Color {
  public Gee.HashMap<Gdk.RGBA?, int> quanitize_pixbuff (Gdk.Pixbuf pixbuf) {
    var histogram = new Gee.HashMap<Gdk.RGBA?, int>
      (
       (a) =>
       ((uint8) a.red * 255 + ((uint8) a.green * 255 << 8) + ((uint8) a.blue * 255 << 16)),
       (a, b) => (a.red == b.red && a.blue == b.blue && a.green == b.green)
      );
    unowned uint8[] pixels = pixbuf.get_pixels ();
    var width = pixbuf.get_width ();
    var height = pixbuf.get_height ();
    var rowstride = pixbuf.get_rowstride ();
    var n_channels = pixbuf.get_n_channels ();

    for (var y = 0; y < height; y++) {
      for (var x = 0; x < width; x++) {
        var offset = y * rowstride + x * n_channels;

        var r = pixels[offset];
        var g = pixels[offset + 1];
        var b = pixels[offset + 2];
        var a = (n_channels == 4) ? pixels[offset + 3] : 255;

        var rgba = Gdk.RGBA () {
          red = r / 255.0,
          green = g / 255.0,
          blue = b / 255.0,
          alpha = a / 255.0
        };

        histogram[rgba] = histogram[rgba] + 1;
      }
    }
    return histogram;
  }

  public Gdk.RGBA ? get_most_common_color (ref Gee.HashMap<Gdk.RGBA?, int> histogram, double? luma = null, double luma_delta = 0.4, bool inv = false) {
    int max = 0;
    Gdk.RGBA? color = null;
    foreach (var kv in histogram) {
      if (kv.value > max) {
        if (luma != null) {
          var _luma = get_pixel_luminance (kv.key);
          var state = Math.fabs (_luma - luma) > luma_delta;
          if (inv ? !state : state)
            continue;
        }
        max = kv.value;
        color = kv.key;
      }
    }
    return color;
  }

  public double get_pixel_luminance (Gdk.RGBA pix) {
    return pix.red * 0.299 + pix.green * 0.587 + pix.blue * 0.114;
  }
}
